@@ -1,6 +1,6 @@
 package Ubic::Daemon;
 BEGIN {
-  $Ubic::Daemon::VERSION = '1.32';
+  $Ubic::Daemon::VERSION = '1.33';
 }
 
 use strict;
@@ -14,6 +14,7 @@ use POSIX qw(setsid);
 use Time::HiRes qw(sleep);
 use Params::Validate qw(:all);
 use Carp;
+use Config;
 
 use Ubic::Lockf;
 use Ubic::Daemon::Status;
@@ -42,6 +43,18 @@ sub import {
     __PACKAGE__->export_to_level(1, @_);
 }
 
+{
+    my @signame;
+    sub _signame {
+        my $signum = shift;
+        unless (@signame) {
+            @signame = split /\s+/, $Config{sig_name};
+        }
+        return $signame[$signum];
+
+    }
+}
+
 sub _log {
     my $fh = shift;
     return unless defined $fh;
@@ -101,9 +114,11 @@ sub start_daemon($) {
         ubic_log => { type => SCALAR, optional => 1 },
         user => { type => SCALAR, optional => 1 },
         term_timeout => { type => SCALAR, default => 10, regex => qr/^\d+$/ },
+        cwd => { type => SCALAR, optional => 1 },
+        env => { type => HASHREF, optional => 1 },
     });
-    my           ($bin, $function, $name, $pidfile, $stdout, $stderr, $ubic_log, $user, $term_timeout)
-    = @options{qw/ bin   function   name   pidfile   stdout   stderr   ubic_log   user term_timeout /};
+    my           ($bin, $function, $name, $pidfile, $stdout, $stderr, $ubic_log, $user, $term_timeout, $cwd, $env)
+    = @options{qw/ bin   function   name   pidfile   stdout   stderr   ubic_log   user   term_timeout   cwd   env /};
     if (not defined $bin and not defined $function) {
         croak "One of 'bin' and 'function' should be specified";
     }
@@ -138,7 +153,7 @@ sub start_daemon($) {
         my $ubic_fh;
         my $lock;
         my $instant_exit = sub {
-            my $status = shift; # nobody cares for this status, anyway...
+            my $status = shift; # nobody cares for this status anyway...
             close($ubic_fh) if $ubic_fh;
             STDOUT->flush;
             STDERR->flush;
@@ -212,7 +227,7 @@ sub start_daemon($) {
                     }
                     _log($ubic_fh, "sending SIGKILL to $child");
                     kill -9 => $child;
-                    _log($ubic_fh, "child probably killed by SIGKILL");
+                    _log($ubic_fh, "daemon $child probably killed by SIGKILL");
                     $pid_state->remove();
                     $instant_exit->(0);
                 };
@@ -236,25 +251,33 @@ sub start_daemon($) {
                 $? = 0;
                 waitpid($child, 0);
                 if ($? > 0) {
-                    my $msg;
-                    my $signal = $? & 127;
-                    if ($signal) {
+                    my $msg = "daemon $child failed with \$? = $?";
+                    if (my $signal = $? & 127) {
                         if ($sigterm_sent && $signal == &POSIX::SIGTERM) {
                             # it's ok, we probably sent this signal ourselves
-                            _log($ubic_fh, "daemon exited by sigterm");
+                            _log($ubic_fh, "daemon $child exited by sigterm");
                             $pid_state->remove;
                             $instant_exit->(0);
                         }
-                        $msg = "Daemon $child failed with signal $signal";
+                        my $signame = _signame($signal);
+                        if (defined $signame) {
+                            $msg = "daemon $child failed with signal $signame ($signal)";
+                        }
+                        else {
+                            $msg = "daemon $child failed with signal $signal";
+                        }
                     }
-                    else {
-                        $msg = "Daemon failed: $?";
+                    elsif ($? & 128) {
+                        $msg = "daemon $child failed, core dumped";
+                    }
+                    elsif (my $code = $? >> 8) {
+                        $msg = "daemon $child failed, exit code $code";
                     }
                     _log($ubic_fh, $msg);
                     $pid_state->remove;
                     $instant_exit->(1);
                 }
-                _log($ubic_fh, "daemon exited");
+                _log($ubic_fh, "daemon $child exited");
                 $pid_state->remove;
                 $instant_exit->(0);
             }
@@ -268,6 +291,15 @@ sub start_daemon($) {
                 setpgrp;
                 $0 = "ubic-daemon $name";
 
+                if (defined $cwd) {
+                    chdir $cwd or die "chdir to '$cwd' failed: $!";
+                }
+                if (defined $env) {
+                    for my $key (keys %{ $env }) {
+                        $ENV{$key} = $env->{$key};
+                    }
+                }
+
                 print {$write_pipe} "execing into daemon\n" or die "Can't write to pipe: $!";
                 close($write_pipe) or die "Can't close pipe: $!";
 
@@ -281,7 +313,6 @@ sub start_daemon($) {
                 else {
                     $function->();
                 }
-
             }
         };
         if ($write_pipe) {
@@ -306,7 +337,14 @@ sub start_daemon($) {
 }
 
 sub check_daemon {
-    my ($pidfile) = @_;
+    my $pidfile = shift;
+    my $options = validate(@_, {
+        quiet => { optional => 1 },
+    });
+
+    my $print = sub {
+        print @_, "\n" unless $options->{quiet};
+    };
 
     my $pid_state = Ubic::Daemon::PidState->new($pidfile);
     return undef if $pid_state->is_empty;
@@ -333,7 +371,7 @@ sub check_daemon {
     }
     unless ($OS->pid_exists($piddata->{daemon})) {
         $pid_state->remove;
-        print "pidfile $pidfile removed - daemon with cached pid $piddata->{daemon} not found\n";
+        $print->("pidfile $pidfile removed - daemon with cached pid $piddata->{daemon} not found");
         return undef;
     }
 
@@ -342,19 +380,19 @@ sub check_daemon {
 
     my $guid = $OS->pid2guid($piddata->{daemon});
     unless ($guid) {
-        print "daemon '$daemon_cmd' from $pidfile just disappeared\n";
+        $print->("daemon '$daemon_cmd' from $pidfile just disappeared");
         return undef;
     }
     if ($guid eq $piddata->{guid}) {
-        print "killing unguarded daemon '$daemon_cmd' with pid $piddata->{daemon} from $pidfile\n";
+        $print->("killing unguarded daemon '$daemon_cmd' with pid $piddata->{daemon} from $pidfile");
         kill -9 => $piddata->{daemon};
         $pid_state->remove;
-        print "pidfile $pidfile removed\n";
+        $print->("pidfile $pidfile removed");
         return undef;
     }
-    print "daemon pid $piddata->{daemon} cached in pidfile $pidfile, ubic-guardian not found\n";
-    print "current process '$daemon_cmd' with that pid looks too fresh and will not be killed\n";
-    print "pidfile $pidfile removed\n";
+    $print->("daemon pid $piddata->{daemon} cached in pidfile $pidfile, ubic-guardian not found");
+    $print->("current process '$daemon_cmd' with that pid looks too fresh and will not be killed");
+    $print->("pidfile $pidfile removed");
     $pid_state->remove;
     return undef;
 }
@@ -371,7 +409,7 @@ Ubic::Daemon - toolkit for creating daemonized process
 
 =head1 VERSION
 
-version 1.32
+version 1.33
 
 =head1 SYNOPSIS
 
@@ -461,10 +499,18 @@ Write all daemon's error output to given file. If not specified, all stderr will
 
 =item I<ubic_log>
 
-Optional filename of ubic log. It will contain some technical information about running daemon.
+Optional filename of ubic log. Log will contain some technical information about running daemon.
 
 If not specified, this logging facility will be disabled.
 
+=item I<cwd>
+
+Change working directory before starting a daemon. Optional.
+
+=item I<env>
+
+Modify environment before starting a daemon. Optional. Must be a plain hashref if specified.
+
 =item I<term_timeout>
 
 Can contain integer number of seconds to wait between sending I<SIGTERM> and I<SIGKILL> to daemon.
@@ -487,9 +533,9 @@ Returns instance of L<Ubic::Daemon::Status> class if daemon is alive, and false
 
 Probably. But it definitely is ready for production usage.
 
-This module currently is Linux-specific, because it uses C</proc> some magic. Patches are very welcome to fix this.
+This module is not compatible with Windows by now. It can be fixed by implementing correct C<Ubic::Daemon::OS::Windows> module.
 
-If you can't figure out why there are C<ubic-guardian> processes in your C<ps> output, see L<Ubic::Manual::FAQ>, answer is there.
+If you wonder why there are C<ubic-guardian> processes in your C<ps> output, see L<Ubic::Manual::FAQ>, answer is there.
 
 =head1 SEE ALSO
 
@@ -497,7 +543,7 @@ L<Ubic::Service::SimpleDaemon> - simplest ubic service which uses Ubic::Daemon
 
 There are also a plenty of other daemonizers on CPAN:
 
-L<MooseX::Daemonize>, L<Proc::Daemon>, L<Daemon::Generic>, L<Net::ServeR::Daemonize>.
+L<MooseX::Daemonize>, L<Proc::Daemon>, L<Daemon::Generic>, L<Net::Server::Daemonize>.
 
 =head1 AUTHOR
 
