@@ -1,6 +1,6 @@
 package Ubic::Service::SimpleDaemon;
 BEGIN {
-  $Ubic::Service::SimpleDaemon::VERSION = '1.32';
+  $Ubic::Service::SimpleDaemon::VERSION = '1.33';
 }
 
 use strict;
@@ -42,6 +42,8 @@ sub new {
         stdout => { type => SCALAR, optional => 1 },
         stderr => { type => SCALAR, optional => 1 },
         ubic_log => { type => SCALAR, optional => 1 },
+        cwd => { type => SCALAR, optional => 1 },
+        env => { type => HASHREF, optional => 1 },
     });
 
     return bless {%$params} => $class;
@@ -55,15 +57,13 @@ sub pidfile {
 
 sub start_impl {
     my ($self) = @_;
+
     my $start_params = {
         pidfile => $self->pidfile,
         bin => $self->{bin},
-        stdout => $self->{stdout} || "/dev/null",
-        stderr => $self->{stderr} || "/dev/null",
-        ubic_log => $self->{ubic_log} || "/dev/null",
     };
-    if ($self->{user}) {
-        $start_params->{user} = $self->{user}; # do we actually need this? Ubic.pm should call setuid for us...
+    for (qw/ env cwd stdout stderr ubic_log /) {
+        $start_params->{$_} = $self->{$_} if defined $self->{$_};
     }
     start_daemon($start_params);
 }
@@ -109,7 +109,7 @@ Ubic::Service::SimpleDaemon - declarative service for daemonizing any binary
 
 =head1 VERSION
 
-version 1.32
+version 1.33
 
 =head1 SYNOPSIS
 
@@ -162,6 +162,18 @@ File into which daemon's stdout will be redirected. Default is C</dev/null>.
 
 File into which daemon's stderr will be redirected. Default is C</dev/null>.
 
+=item I<ubic_log>
+
+Optional filename of ubic log. Log will contain some technical information about running daemon.
+
+=item I<cwd>
+
+Change working directory before starting a daemon. Optional.
+
+=item I<env>
+
+Modify environment before starting a daemon. Optional. Must be a plain hashref if specified.
+
 =item I<name>
 
 Service's name.
